package com.sd.business.controller.board;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sd.business.entity.board.dto.DailyBoardSelectDto;
import com.sd.business.entity.board.dto.SalesBoardSelectDto;
import com.sd.business.entity.board.vo.*;
import com.sd.business.service.board.SalesBoardService;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 销售看板 前端控制器
 * </p>
 *
 * @author
 * @since 2023-11-16
 */
@RestController
@RequestMapping("/salesBoard")
public class SalesBoardController {

    @Autowired
    private SalesBoardService salesBoardService;

    /**
     * 获取sku销量排名列表分页
     * @return
     */
    @PostMapping("/getSkuSalesRankingPage")
    public Page<SkuSalesRankingVo> getSkuSalesRankingPage(@RequestBody SalesBoardSelectDto dto) {
        return salesBoardService.getSkuSalesRankingPage(dto);
    }

    /**
     * sku销量排名列表导出
     * @return
     */
    @PostMapping("/skuSalesRankingExportExcel")
    public void skuSalesRankingExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<SkuSalesRankingVo> page = salesBoardService.getSkuSalesRankingPage(dto);
        List<SkuSalesRankingVo> list = page.getRecords();
        ExcelUtil.export(response,"sku销量排行", "sku销量排行数据", list, SkuSalesRankingVo.class);
    }

    /**
     * 获取sku销售额排名列表分页
     * @return
     */
    @PostMapping("/getSkuSalesAmountRankingPage")
    public Page<SkuSalesAmountRankingVo> getSkuSalesAmountRankingPage(@RequestBody SalesBoardSelectDto dto) {
        return salesBoardService.getSkuSalesAmountRankingPage(dto);
    }

    /**
     * sku销售额排名列表导出
     * @return
     */
    @PostMapping("/skuSalesAmountRankingExportExcel")
    public void skuSalesAmountRankingExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<SkuSalesAmountRankingVo> page = salesBoardService.getSkuSalesAmountRankingPage(dto);
        List<SkuSalesAmountRankingVo> list = page.getRecords();
        ExcelUtil.export(response,"sku销售额排行", "sku销售额排行数据", list, SkuSalesAmountRankingVo.class);
    }

    /**
     * 获取bom销量排名列表分页
     * @return
     */
    @PostMapping("/getBomSalesRankingPage")
    public Page<BomSalesRankingVo> getBomSalesRankingPage(@RequestBody SalesBoardSelectDto dto) {
        return salesBoardService.getBomSalesRankingPage(dto);
    }

    /**
     * bom销量排名列表导出
     * @return
     */
    @PostMapping("/bomSalesRankingExportExcel")
    public void bomSalesRankingExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<BomSalesRankingVo> page = salesBoardService.getBomSalesRankingPage(dto);
        List<BomSalesRankingVo> list = page.getRecords();
        ExcelUtil.export(response,"bom销量排行", "bom销量排行数据", list, BomSalesRankingVo.class);
    }

    /**
     * 获取bom销售额排名列表分页
     * @return
     */
    @PostMapping("/getBomSalesAmountRankingPage")
    public Page<BomSalesAmountRankingVo> getBomSalesAmountRankingPage(@RequestBody SalesBoardSelectDto dto) {
        return salesBoardService.getBomSalesAmountRankingPage(dto);
    }

    /**
     * bom销售额排名列表导出
     * @return
     */
    @PostMapping("/bomSalesAmountRankingExportExcel")
    public void bomSalesAmountRankingExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<BomSalesAmountRankingVo> page = salesBoardService.getBomSalesAmountRankingPage(dto);
        List<BomSalesAmountRankingVo> list = page.getRecords();
        ExcelUtil.export(response,"bom销售额排行", "bom销售额排行数据", list, BomSalesAmountRankingVo.class);
    }


    /**
     * 获取sku销售地区排名列表分页
     * @return
     */
    @PostMapping("/getSkuSalesRegionRankingPage")
    public Page<SkuSalesRegionRankingVo> getSkuSalesRegionRankingPage(@RequestBody SalesBoardSelectDto dto) {
        return salesBoardService.getSkuSalesRegionRankingPage(dto);
    }

    /**
     * sku销售地区排名列表导出
     * @return
     */
    @PostMapping("/skuSalesRegionRankingExportExcel")
    public void skuSalesRegionRankingExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<SkuSalesRegionRankingVo> page = salesBoardService.getSkuSalesRegionRankingPage(dto);
        List<SkuSalesRegionRankingVo> list = page.getRecords();
        ExcelUtil.export(response,"sku销售地区排行", "sku销售地区排行数据", list, SkuSalesRegionRankingVo.class);
    }

    /**
     * 获取sku销量增长率排名列表分页
     * @return
     */
    @PostMapping("/getSkuSalesGrowthRateRankingPage")
    public Page<SkuSalesGrowthRateRankingVo> getSkuSalesGrowthRateRankingPage(@RequestBody SalesBoardSelectDto dto) {
        return salesBoardService.getSkuSalesGrowthRateRankingPage(dto);
    }

    /**
     * sku销量增长率排名列表导出
     * @return
     */
    @PostMapping("/skuSalesGrowthRateRankingExportExcel")
    public void skuSalesGrowthRateRankingExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<SkuSalesGrowthRateRankingVo> page = salesBoardService.getSkuSalesGrowthRateRankingPage(dto);
        List<SkuSalesGrowthRateRankingVo> list = page.getRecords();
        ExcelUtil.export(response,"sku销量增长率排行", "sku销量增长率排行数据", list, SkuSalesGrowthRateRankingVo.class);
    }

    /**
     * 各事业部bom销量统计列表
     * @return
     */
    @PostMapping("/getDepartmentBomSalesStatisticsList")
    public List<DepartmentBomSalesBoardVo> getDepartmentBomSalesStatisticsList(@Validated @RequestBody DailyBoardSelectDto dto) {
        return salesBoardService.getDepartmentBomSalesStatisticsList(dto);
    }
}
