package com.sd.business.service.order.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.board.bo.OrderSalesShipmentStatisticsBo;
import com.sd.business.entity.board.dto.TurnoverRateBoardSelectDto;
import com.sd.business.entity.board.vo.TurnoverRateStatisticsVo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.po.InventoryBackup;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderSalesShipmentStatisticsMapper;
import com.sd.business.service.inventory.InventoryBackupService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderSalesShipmentStatisticsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.Comparator.comparing;

/**
 * <p>
 * 订单销售出库统计 服务实现类
 * </p>
 *
 * @author
 * @since 2023-10-20
 */
@Service
public class OrderSalesShipmentStatisticsServiceImpl implements OrderSalesShipmentStatisticsService {

    @Resource
    private OrderSalesShipmentStatisticsMapper orderSalesShipmentStatisticsMapper;

    @Autowired
    private InventoryBackupService inventoryBackupService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @Override
    public Map<Long, OrderSalesShipmentStatisticsBo> getSalesShipmentStatisticsByDate(List<Long> bomSpecIds) {
        Date date = new Date();
        Date startDate = DateUtil.beginOfDay(DateUtil.offsetDay(date, -90));
        IWrapper<OrderSku> wrapper = IWrapper.getWrapper();
        wrapper.orderByDesc("os", OrderSku::getId);
        wrapper.between("oi", OrderInfo::getShippingTime, startDate, date);
        wrapper.groupBy("bs.id");
        List<OrderSalesShipmentStatisticsBo> list = orderSalesShipmentStatisticsMapper.getOrderSalesShipmentStatisticsList(wrapper);
        return list.stream().collect(Collectors.toMap(OrderSalesShipmentStatisticsBo::getBomSpecId, Function.identity()));
    }

    @Override
    public List<TurnoverRateStatisticsVo> getTurnoverRateStatisticsList(TurnoverRateBoardSelectDto dto) {
        Date date = new Date();
        String ninetyDaysDate = DateUtil.formatDate(DateUtil.offsetDay(date, -90));
        String sixtyDaysDate = DateUtil.formatDate(DateUtil.offsetDay(date, -60));
        String thirtyDaysDate = DateUtil.formatDate(DateUtil.offsetDay(date, -30));

        IWrapper<OrderSku> wrapper = IWrapper.getWrapper();
        wrapper.orderByDesc("os", OrderSku::getId);
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());
        wrapper.between("oi", OrderInfo::getShippingTime, ninetyDaysDate, date);
        if (Objects.equals(dto.getBomClassify(), 1)) {
            wrapper.eq("bc", BomClassify::getCode, "201.G");
        } else if (Objects.equals(dto.getBomClassify(), 2)){
            wrapper.eq("bc", BomClassify::getCode,"202.G");
        } else if (Objects.equals(dto.getBomClassify(), 3)) {
            wrapper.eq("bc", BomClassify::getCode,"203.G");
        }
        wrapper.ne("bs", BomSpec::getCode, "1010000007");
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.like("bs", BomSpec::getWidth, dto.getWidth());
        wrapper.like("bs", BomSpec::getHeight, dto.getHeight());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.groupBy("bs.id");
        List<OrderSalesShipmentStatisticsBo> statisticsVoList = orderSalesShipmentStatisticsMapper.getOrderSalesShipmentStatisticsList(wrapper);
        if (ObjectUtil.isEmpty(statisticsVoList)) {
            return Collections.emptyList();
        }
        List<Long> bomSpecIds = statisticsVoList.stream().map(OrderSalesShipmentStatisticsBo::getBomSpecId).collect(Collectors.toList());

        Map<String, String> bomChromatophoreMap = dictCommonDataService.list(q -> q.eq(DictCommonData::getDictCode, "bom_chromatophore"))
                .stream().collect(Collectors.toMap(DictCommonData::getDictKey, DictCommonData::getDictValue));

        // 查询前90、60、30天的所有库存数据
        List<InventoryBackup> inventoryBackupList = inventoryBackupService.list(q -> q
                .in(InventoryBackup::getBackupDate, ninetyDaysDate, sixtyDaysDate, thirtyDaysDate)
                .eq(InventoryBackup::getDepartmentId, DepartmentConstant.SD_SPORTS)
                .eq(InventoryBackup::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT)
                .in(InventoryBackup::getBomSpecId, bomSpecIds));

        // 期初库存数据
        Map<String, Map<Long, InventoryBackup>> beginBackupMap = inventoryBackupList.stream()
                .collect(
                        Collectors.groupingBy(
                                item -> DateUtil.format(item.getBackupDate(), "yyyy-MM-dd"),
                                Collectors.toMap(InventoryBackup::getBomSpecId, Function.identity())));

        // 期末库存数据
        Map<Long, Inventory> endBackupMap = inventoryService.mapKEntity(Inventory::getBomSpecId, q -> q
                .eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
                .eq(Inventory::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT)
                .in(Inventory::getBomSpecId, bomSpecIds));

        List<TurnoverRateStatisticsVo> list = statisticsVoList.stream()
                .map(item -> {
                    Long bomSpecId = item.getBomSpecId();
                    // 计算周转率
                    // 时间段库存周转天数 = 时间段天数 * (1 / 2) * (期初库存数量+期末库存数量) / 时间段销售量;
                    // 库存周转率 = 时间段天数 / 库存周转天数。
                    // 获取期初和期末数据
                    InventoryBackup beginBackupNinetyDays = beginBackupMap.getOrDefault(ninetyDaysDate, Collections.emptyMap()).get(bomSpecId);
                    InventoryBackup beginBackupSixtyDays = beginBackupMap.getOrDefault(sixtyDaysDate, Collections.emptyMap()).get(bomSpecId);
                    InventoryBackup beginBackupThirtyDays = beginBackupMap.getOrDefault(thirtyDaysDate, Collections.emptyMap()).get(bomSpecId);
                    Inventory endBackup = endBackupMap.get(bomSpecId);
                    // 计算bom近90、60、30天的期初库存数量
                    BigDecimal beginQuantityNinetyDays = BigDecimal.ZERO;
                    BigDecimal beginQuantitySixtyDays = BigDecimal.ZERO;
                    BigDecimal beginQuantityThirtyDays = BigDecimal.ZERO;
                    BigDecimal endQuantity = BigDecimal.ZERO;
                    if (beginBackupNinetyDays != null) {
                        beginQuantityNinetyDays = beginBackupNinetyDays.getQuantity()
                                .add(beginBackupNinetyDays.getLockQuantity() == null ? BigDecimal.ZERO : beginBackupNinetyDays.getLockQuantity());
                    }
                    if (beginBackupSixtyDays != null) {
                        beginQuantitySixtyDays = beginBackupSixtyDays.getQuantity()
                                .add(beginBackupSixtyDays.getLockQuantity() == null ? BigDecimal.ZERO : beginBackupSixtyDays.getLockQuantity());
                    }
                    if (beginBackupThirtyDays != null) {
                        beginQuantityThirtyDays = beginBackupThirtyDays.getQuantity()
                                .add(beginBackupThirtyDays.getLockQuantity() == null ? BigDecimal.ZERO : beginBackupThirtyDays.getLockQuantity());
                    }
                    if (endBackup != null) {
                        endQuantity = endBackup.getQuantity().add(endBackup.getLockQuantity() == null ? BigDecimal.ZERO : endBackup.getLockQuantity());
                    }
                    // 时间段天数
                    BigDecimal days;
                    BigDecimal turnoverRateNinetyDays = BigDecimal.ZERO;
                    BigDecimal turnoverRateSixtyDays = BigDecimal.ZERO;
                    BigDecimal turnoverRateThirtyDays = BigDecimal.ZERO;
                    if (ObjectUtil.notEqual(item.getNinetyDaysSalesQuantity(), BigDecimal.ZERO)
                            && ObjectUtil.notEqual(beginQuantityNinetyDays.add(endQuantity), BigDecimal.ZERO)) {
                        days = new BigDecimal(90);
                        turnoverRateNinetyDays = days
                                .divide(days.multiply(new BigDecimal("0.5"))
                                                .multiply(beginQuantityNinetyDays.add(endQuantity))
                                                .divide(item.getNinetyDaysSalesQuantity(), 4, RoundingMode.HALF_UP),
                                        4, RoundingMode.HALF_UP);
                    }
                    if (ObjectUtil.notEqual(item.getSixtyDaysSalesQuantity(), BigDecimal.ZERO)
                            && ObjectUtil.notEqual(beginQuantitySixtyDays.add(endQuantity), BigDecimal.ZERO)) {
                        days = new BigDecimal(60);
                        turnoverRateSixtyDays = days
                                .divide(days.multiply(new BigDecimal("0.5"))
                                                .multiply(beginQuantitySixtyDays.add(endQuantity))
                                                .divide(item.getSixtyDaysSalesQuantity(), 4, RoundingMode.HALF_UP),
                                        4, RoundingMode.HALF_UP);
                    }
                    if (ObjectUtil.notEqual(item.getThirtyDaysSalesQuantity(), BigDecimal.ZERO)
                            && ObjectUtil.notEqual(beginQuantityThirtyDays.add(endQuantity), BigDecimal.ZERO)) {
                        days = new BigDecimal(30);
                        turnoverRateThirtyDays = days
                                .divide(days.multiply(new BigDecimal("0.5"))
                                                .multiply(beginQuantityThirtyDays.add(endQuantity))
                                                .divide(item.getThirtyDaysSalesQuantity(), 4, RoundingMode.HALF_UP),
                                        4, RoundingMode.HALF_UP);
                    }
                    if (ObjectUtil.equals(endQuantity, BigDecimal.ZERO)) {
                        return null;
                    }
                    TurnoverRateStatisticsVo vo = new TurnoverRateStatisticsVo();
                    vo.setBomSpecCode(item.getBomSpecCode());
                    vo.setBomSpecName(item.getBomSpecName());
                    vo.setWidth(item.getWidth());
                    vo.setHeight(item.getHeight());
                    vo.setChromatophore(bomChromatophoreMap.get(item.getChromatophore()));
                    vo.setSalesQuantityThirtyDays(item.getThirtyDaysSalesQuantity());
                    vo.setSalesQuantitySixtyDays(item.getSixtyDaysSalesQuantity());
                    vo.setSalesQuantityNinetyDays(item.getNinetyDaysSalesQuantity());
                    vo.setTurnoverRateNinetyDays(turnoverRateNinetyDays);
                    vo.setTurnoverRateSixtyDays(turnoverRateSixtyDays);
                    vo.setTurnoverRateThirtyDays(turnoverRateThirtyDays);
                    vo.setQuantity(endQuantity);
                    return vo;
                }).filter(Objects::nonNull)
                .sorted(comparing(TurnoverRateStatisticsVo::getTurnoverRateNinetyDays))
                .collect(Collectors.toList());
        return list;
    }

    @Override
    public Map<Long, String> getTurnoverRateMapByBomSpecIds(List<Long> bomSpecIds) {
        IWrapper<OrderSku> wrapper = IWrapper.getWrapper();
        wrapper.in("oq.bom_spec_id", bomSpecIds);
        wrapper.groupBy("oq.bom_spec_id");
        List<OrderSalesShipmentStatisticsBo> list = orderSalesShipmentStatisticsMapper.getTurnoverRateMapByBomSpecIds(wrapper);
        return list.stream().collect(Collectors.toMap(OrderSalesShipmentStatisticsBo::getBomSpecId, OrderSalesShipmentStatisticsBo::getTurnoverRate));
    }
}
